var Cursor, DB, RestClient, RpcClient, http, util;
var __slice = Array.prototype.slice;
http = require('http');
util = require('util');
Cursor = require('./cursor');
RestClient = require('./rest_client');
RpcClient = require('./rpc_client');
DB = (function() {
  function DB() {}
  DB.prototype.open = function(host, port) {
    this.host = host != null ? host : 'localhost';
    this.port = port != null ? port : 1978;
    this.client = http.createClient(this.port, this.host);
    return this;
  };
  DB.prototype.close = function(callback) {
    var request;
    request = this.client.request('GET', '/rpc/echo', {
      'Connection': 'close'
    });
    request.end();
    return request.on('end', function() {
      return callback();
    });
  };
  DB.prototype.echo = function(input, callback) {
    return RpcClient.call(this.client, 'echo', input, function(error, status, output) {
      if (error != null) {
        return callback(error, output);
      } else if (status !== 200) {
        error = new Error("Unexpected response from server: " + status);
        return callback(error, output);
      } else {
        return callback(void 0, output);
      }
    });
  };
  DB.prototype.report = function(callback) {
    return RpcClient.call(this.client, 'report', {}, function(error, status, output) {
      if (error != null) {
        return callback(error, output);
      } else if (status !== 200) {
        error = new Error("Unexpected response from server: " + status);
        return callback(error, output);
      } else {
        return callback(void 0, output);
      }
    });
  };
  DB.prototype.status = function() {
    var args, callback, database, rpc_args;
    args = 1 <= arguments.length ? __slice.call(arguments, 0) : [];
    switch (args.length) {
      case 1:
        callback = args[0];
        break;
      case 2:
        database = args[0];
        callback = args[1];
        break;
      default:
        throw new Error("Invalid number of arguments (" + args.length + ") to status");
    }
    rpc_args = {};
    if (database != null) {
      rpc_args.DB = database;
    }
    return RpcClient.call(this.client, 'status', rpc_args, function(error, status, output) {
      if (error != null) {
        return callback(error, output);
      } else if (status !== 200) {
        error = new Error("Unexpected response from server: " + status);
        return callback(error, output);
      } else {
        return callback(void 0, output);
      }
    });
  };
  DB.prototype.clear = function() {
    var args, callback, database, rpc_args;
    args = 1 <= arguments.length ? __slice.call(arguments, 0) : [];
    switch (args.length) {
      case 1:
        callback = args[0];
        break;
      case 2:
        database = args[0];
        callback = args[1];
        break;
      default:
        throw new Error("Invalid number of arguments (" + args.length + ") to clear");
    }
    rpc_args = {};
    if (database != null) {
      rpc_args.DB = database;
    }
    return RpcClient.call(this.client, 'clear', rpc_args, function(error, status, output) {
      if (error != null) {
        return callback(error, output);
      } else if (status !== 200) {
        error = new Error("Unexpected response from server: " + status);
        return callback(error, output);
      } else {
        return callback(void 0, output);
      }
    });
  };
  DB.prototype.set = function() {
    var args, callback, database, key, value;
    key = arguments[0], value = arguments[1], args = 3 <= arguments.length ? __slice.call(arguments, 2) : [];
    switch (args.length) {
      case 1:
        callback = args[0];
        break;
      case 2:
        database = args[0];
        callback = args[1];
        break;
      default:
        throw new Error("Invalid number of arguments (" + args.length + ") to set");
    }
    return RestClient.put(this.client, key, value, function(error) {
      return callback(error);
    });
  };
  DB.prototype.add = function() {
    var args, callback, database, key, rpc_args, value;
    key = arguments[0], value = arguments[1], args = 3 <= arguments.length ? __slice.call(arguments, 2) : [];
    switch (args.length) {
      case 1:
        callback = args[0];
        break;
      case 2:
        database = args[0];
        callback = args[1];
        break;
      default:
        throw new Error("Invalid number of arguments (" + args.length + ") to add");
    }
    rpc_args = {
      key: key,
      value: value
    };
    if (database != null) {
      rpc_args.DB = database;
    }
    return RpcClient.call(this.client, 'add', rpc_args, function(error, status, output) {
      if (error != null) {
        return callback(error, output);
      } else if (status === 200) {
        return callback(void 0, output);
      } else if (status === 450) {
        return callback(new Error("Record exists"), output);
      } else {
        return callback(new Error("Unexpected response from server: " + status), output);
      }
    });
  };
  DB.prototype.replace = function() {
    var args, callback, database, key, rpc_args, value;
    key = arguments[0], value = arguments[1], args = 3 <= arguments.length ? __slice.call(arguments, 2) : [];
    switch (args.length) {
      case 1:
        callback = args[0];
        break;
      case 2:
        database = args[0];
        callback = args[1];
        break;
      default:
        throw new Error("Invalid number of arguments (" + args.length + ") to replace");
    }
    rpc_args = {
      key: key,
      value: value
    };
    if (database != null) {
      rpc_args.DB = database;
    }
    return RpcClient.call(this.client, 'replace', rpc_args, function(error, status, output) {
      if (error != null) {
        return callback(error, output);
      } else if (status === 200) {
        return callback(void 0, output);
      } else if (status === 450) {
        return callback(new Error("Record does not exist"), output);
      } else {
        return callback(new Error("Unexpected response from server: " + status), output);
      }
    });
  };
  DB.prototype.append = function() {
    var args, callback, database, key, rpc_args, value;
    key = arguments[0], value = arguments[1], args = 3 <= arguments.length ? __slice.call(arguments, 2) : [];
    switch (args.length) {
      case 1:
        callback = args[0];
        break;
      case 2:
        database = args[0];
        callback = args[1];
        break;
      default:
        throw new Error("Invalid number of arguments (" + args.length + ") to append");
    }
    rpc_args = {
      key: key,
      value: value
    };
    if (database != null) {
      rpc_args.DB = database;
    }
    return RpcClient.call(this.client, 'append', rpc_args, function(error, status, output) {
      if (error != null) {
        return callback(error, output);
      } else if (status !== 200) {
        error = new Error("Unexpected response from server: " + status);
        return callback(error, output);
      } else {
        return callback(void 0, output);
      }
    });
  };
  DB.prototype.increment = function() {
    var args, callback, database, key, num, rpc_args;
    key = arguments[0], num = arguments[1], args = 3 <= arguments.length ? __slice.call(arguments, 2) : [];
    switch (args.length) {
      case 1:
        callback = args[0];
        break;
      case 2:
        database = args[0];
        callback = args[1];
        break;
      default:
        throw new Error("Invalid number of arguments (" + args.length + ") to increment");
    }
    rpc_args = {
      key: key,
      num: num
    };
    if (database != null) {
      rpc_args.DB = database;
    }
    return RpcClient.call(this.client, 'increment', rpc_args, function(error, status, output) {
      if (error != null) {
        return callback(error, output);
      } else if (status === 200) {
        return callback(void 0, output);
      } else if (status === 450) {
        return callback(new Error("The existing record was not compatible"), output);
      } else {
        return callback(new Error("Unexpected response from server: " + status), output);
      }
    });
  };
  DB.prototype.incrementDouble = function() {
    var args, callback, database, key, num, rpc_args;
    key = arguments[0], num = arguments[1], args = 3 <= arguments.length ? __slice.call(arguments, 2) : [];
    switch (args.length) {
      case 1:
        callback = args[0];
        break;
      case 2:
        database = args[0];
        callback = args[1];
        break;
      default:
        throw new Error("Invalid number of arguments (" + args.length + ") to incrementDouble");
    }
    rpc_args = {
      key: key,
      num: num
    };
    if (database != null) {
      rpc_args.DB = database;
    }
    return RpcClient.call(this.client, 'increment_double', rpc_args, function(error, status, output) {
      if (error != null) {
        return callback(error, output);
      } else if (status === 200) {
        return callback(void 0, output);
      } else if (status === 450) {
        return callback(new Error("The existing record was not compatible"), output);
      } else {
        return callback(new Error("Unexpected response from server: " + status), output);
      }
    });
  };
  DB.prototype.cas = function() {
    var args, callback, database, key, nval, oval, rpc_args;
    key = arguments[0], oval = arguments[1], nval = arguments[2], args = 4 <= arguments.length ? __slice.call(arguments, 3) : [];
    switch (args.length) {
      case 1:
        callback = args[0];
        break;
      case 2:
        database = args[0];
        callback = args[1];
        break;
      default:
        throw new Error("Invalid number of arguments (" + args.length + ") to cas");
    }
    rpc_args = {
      key: key,
      oval: oval
    };
    if (nval != null) {
      rpc_args.nval = nval;
    }
    if (database != null) {
      rpc_args.DB = database;
    }
    return RpcClient.call(this.client, 'cas', rpc_args, function(error, status, output) {
      if (error != null) {
        return callback(error, output);
      } else if (status === 200) {
        return callback(void 0, output);
      } else if (status === 450) {
        return callback(new Error("Record has changed"), output);
      } else {
        return callback(new Error("Unexpected response from server: " + status), output);
      }
    });
  };
  DB.prototype.remove = function() {
    var args, callback, database, key;
    key = arguments[0], args = 2 <= arguments.length ? __slice.call(arguments, 1) : [];
    switch (args.length) {
      case 1:
        callback = args[0];
        break;
      case 2:
        database = args[0];
        callback = args[1];
        break;
      default:
        throw new Error("Invalid number of arguments (" + args.length + ") to remove");
    }
    return RestClient["delete"](this.client, key, function(error) {
      return callback(error);
    });
  };
  DB.prototype.get = function() {
    var args, callback, database, key;
    key = arguments[0], args = 2 <= arguments.length ? __slice.call(arguments, 1) : [];
    switch (args.length) {
      case 1:
        callback = args[0];
        break;
      case 2:
        database = args[0];
        callback = args[1];
        break;
      default:
        throw new Error("Invalid number of arguments (" + args.length + ") to get");
    }
    return RestClient.get(this.client, key, function(error, value) {
      return callback(error, value);
    });
  };
  DB.prototype.setBulk = function() {
    var args, callback, database, key, records, rpc_args, value;
    records = arguments[0], args = 2 <= arguments.length ? __slice.call(arguments, 1) : [];
    switch (args.length) {
      case 1:
        callback = args[0];
        break;
      case 2:
        database = args[0];
        callback = args[1];
        break;
      default:
        throw new Error("Invalid number of arguments (" + args.length + ") to setBulk");
    }
    rpc_args = {};
    if (database != null) {
      rpc_args.DB = database;
    }
    for (key in records) {
      value = records[key];
      rpc_args["_" + key] = value;
    }
    return RpcClient.call(this.client, 'set_bulk', rpc_args, function(error, status, output) {
      if (error != null) {
        return callback(error, output);
      } else if (status === 200) {
        return callback(void 0, output);
      } else {
        return callback(new Error("Unexpected response from server: " + status), output);
      }
    });
  };
  DB.prototype.removeBulk = function() {
    var args, callback, database, key, records, rpc_args, value;
    records = arguments[0], args = 2 <= arguments.length ? __slice.call(arguments, 1) : [];
    switch (args.length) {
      case 1:
        callback = args[0];
        break;
      case 2:
        database = args[0];
        callback = args[1];
        break;
      default:
        throw new Error("Invalid number of arguments (" + args.length + ") to removeBulk");
    }
    rpc_args = {};
    if (database != null) {
      rpc_args.DB = database;
    }
    for (key in records) {
      value = records[key];
      rpc_args["_" + key] = '';
    }
    return RpcClient.call(this.client, 'remove_bulk', rpc_args, function(error, status, output) {
      if (error != null) {
        return callback(error, output);
      } else if (status === 200) {
        return callback(void 0, output);
      } else {
        return callback(new Error("Unexpected response from server: " + status), output);
      }
    });
  };
  DB.prototype.getBulk = function() {
    var args, callback, database, key, keys, rpc_args, _i, _len;
    keys = arguments[0], args = 2 <= arguments.length ? __slice.call(arguments, 1) : [];
    switch (args.length) {
      case 1:
        callback = args[0];
        break;
      case 2:
        database = args[0];
        callback = args[1];
        break;
      default:
        throw new Error("Invalid number of arguments (" + args.length + ") to getBulk");
    }
    rpc_args = {};
    if (database != null) {
      rpc_args.DB = database;
    }
    for (_i = 0, _len = keys.length; _i < _len; _i++) {
      key = keys[_i];
      rpc_args["_" + key] = '';
    }
    return RpcClient.call(this.client, 'get_bulk', rpc_args, function(error, status, output) {
      var key, results, value;
      if (error != null) {
        return callback(error, output);
      } else if (status !== 200) {
        error = new Error("Unexpected response from server: " + status);
        return callback(error, output);
      }
      results = {};
      for (key in output) {
        value = output[key];
        if (key.length > 0 && key[0] === '_') {
          results[key.slice(1, key.length)] = value;
        }
      }
      return callback(void 0, results);
    });
  };
  DB.prototype._matchUsing = function(procedure, pattern, args) {
    var callback, database, max, rpc_args;
    switch (args.length) {
      case 1:
        callback = args[0];
        break;
      case 2:
        max = args[0];
        callback = args[1];
        break;
      case 3:
        max = args[0];
        database = args[2];
        callback = args[1];
        break;
      default:
        throw new Error("Invalid number of arguments (" + args.length + ") to " + procedure);
    }
    rpc_args = {};
    if (procedure === 'match_prefix') {
      rpc_args.prefix = pattern;
    } else {
      rpc_args.regex = pattern;
    }
    if (database != null) {
      rpc_args.DB = database;
    }
    if (max != null) {
      rpc_args.max = max;
    }
    return RpcClient.call(this.client, procedure, rpc_args, function(error, status, output) {
      var key, results, value;
      if (error != null) {
        return callback(error, output);
      } else if (status !== 200) {
        error = new Error("Unexpected response from server: " + status);
        return callback(error, output);
      }
      results = [];
      for (key in output) {
        value = output[key];
        if (key.length > 0 && key[0] === '_') {
          results.push(key.slice(1, key.length));
        }
      }
      return callback(void 0, results);
    });
  };
  DB.prototype.matchPrefix = function() {
    var args, prefix;
    prefix = arguments[0], args = 2 <= arguments.length ? __slice.call(arguments, 1) : [];
    return this._matchUsing('match_prefix', prefix, args);
  };
  DB.prototype.matchRegex = function() {
    var args, regex;
    regex = arguments[0], args = 2 <= arguments.length ? __slice.call(arguments, 1) : [];
    return this._matchUsing('match_regex', regex, args);
  };
  DB.prototype.getCursor = function() {
    var args, callback, cursor, key;
    args = 1 <= arguments.length ? __slice.call(arguments, 0) : [];
    switch (args.length) {
      case 1:
        callback = args[0];
        break;
      case 2:
        key = args[0];
        callback = args[1];
        break;
      default:
        throw new Error("Invalid number of arguments (" + args.length + ") to getCursor");
    }
    cursor = new Cursor(this);
    return cursor.jump(key, function(error) {
      return callback(error, cursor);
    });
  };
  return DB;
})();
module.exports = DB;