module.exports = {
  get: function(client, key, callback) {
    var request;
    request = client.request('GET', "/" + (escape(key)), {
      'Content-Length': 0,
      'Connection': 'keep-alive'
    });
    request.end();
    return request.on('response', function(response) {
      var content_length, offset, value;
      content_length = parseInt(response.headers['content-length'], 10);
      value = new Buffer(content_length);
      offset = 0;
      response.on('data', function(chunk) {
        chunk.copy(value, offset, 0);
        return offset += chunk.length;
      });
      return response.on('end', function() {
        switch (response.statusCode) {
          case 200:
            return callback(void 0, value);
          case 404:
            return callback(void 0, null);
          default:
            return callback(new Error("Unexpected response from server: " + response.statusCode));
        }
      });
    });
  },
  head: function(client, key, callback) {
    var request;
    request = client.request('HEAD', "/" + (escape(key)), {
      'Content-Length': 0,
      'Connection': 'keep-alive'
    });
    request.end();
    return request.on('response', function(response) {
      return response.on('end', function() {
        switch (response.statusCode) {
          case 200:
            return callback(void 0, response.headers);
          case 404:
            return callback(void 0, null);
          default:
            return callback(new Error("Unexpected response from server: " + response.statusCode));
        }
      });
    });
  },
  put: function(client, key, value, callback) {
    var request;
    request = client.request('PUT', "/" + (escape(key)), {
      'Content-Length': value.length,
      'Connection': 'keep-alive '
    });
    request.end(value);
    return request.on('response', function(response) {
      return response.on('end', function() {
        switch (response.statusCode) {
          case 201:
            return callback();
          default:
            return callback(new Error("Unexpected response from server: " + response.statusCode));
        }
      });
    });
  },
  "delete": function(client, key, callback) {
    var request;
    request = client.request('DELETE', "/" + (escape(key)), {
      'Content-Length': 0,
      'Connection': 'keep-alive'
    });
    request.end();
    return request.on('response', function(response) {
      return response.on('end', function() {
        switch (response.statusCode) {
          case 204:
            return callback();
          case 404:
            return callback(new Error("Record not found"));
          default:
            return callback(new Error("Unexpected response from server: " + response.statusCode));
        }
      });
    });
  }
};